/*
 * Copyright 1998-2002 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#ifndef _SYS_VMEM_H
#define	_SYS_VMEM_H

#pragma ident	"@(#)vmem.h	1.5	02/10/25 SMI"

#include <sys/types.h>

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * Per-allocation flags
 */
#define	VM_SLEEP	0x00000000	/* same as KM_SLEEP */
#define	VM_NOSLEEP	0x00000001	/* same as KM_NOSLEEP */
#define	VM_PANIC	0x00000002	/* same as KM_PANIC */
#define	VM_KMFLAGS	0x000000ff	/* flags that must match KM_* flags */

#define	VM_BESTFIT	0x00000100
#define	VM_FIRSTFIT	0x00000200
#define	VM_NEXTFIT	0x00000400

#define	VM_FLAGS	0x0000FFFF

/*
 * Arena creation flags
 */
#define	VMC_POPULATOR	0x00010000

#define	VMC_FLAGS	0xFFFF0000

/*
 * Public segment types
 */
#define	VMEM_ALLOC	0x01
#define	VMEM_FREE	0x02

/*
 * Implementation-private segment types
 */
#define	VMEM_SPAN	0x10
#define	VMEM_ROTOR	0x20
#define	VMEM_WALKER	0x40

/*
 * VMEM_REENTRANT indicates to vmem_walk() that the callback routine may
 * call back into the arena being walked, so vmem_walk() must drop the
 * arena lock before each callback.  The caveat is that since the arena
 * isn't locked, its state can change.  Therefore it is up to the callback
 * routine to handle cases where the segment isn't of the expected type.
 * For example, we use this to walk heap_arena when generating a crash dump;
 * see segkmem_dump() for sample usage.
 */
#define	VMEM_REENTRANT	0x80000000

typedef struct vmem vmem_t;
typedef void *(vmem_alloc_t)(vmem_t *, size_t, int);
typedef void (vmem_free_t)(vmem_t *, void *, size_t);

#ifdef _KERNEL
extern vmem_t *vmem_init(const char *, void *, size_t, size_t,
    vmem_alloc_t *, vmem_free_t *);
extern void vmem_update(void *);
extern size_t vmem_seg_size;
#endif

extern vmem_t *vmem_create(const char *, void *, size_t, size_t,
    vmem_alloc_t *, vmem_free_t *, vmem_t *, size_t, int);
extern void vmem_destroy(vmem_t *);
extern void *vmem_alloc(vmem_t *, size_t, int);
extern void *vmem_xalloc(vmem_t *, size_t, size_t, size_t, size_t,
    void *, void *, int);
extern void vmem_free(vmem_t *, void *, size_t);
extern void vmem_xfree(vmem_t *, void *, size_t);
extern void *vmem_add(vmem_t *, void *, size_t, int);
extern int vmem_contains(vmem_t *, void *, size_t);
extern void vmem_walk(vmem_t *, int, void (*)(void *, void *, size_t), void *);
extern size_t vmem_size(vmem_t *, int);

#ifdef	__cplusplus
}
#endif

#endif	/* _SYS_VMEM_H */
